<?php

/**
 * @file
 * Delivery time slots administration menu items and settings.
 *
 */

/**
 * Display the delivery time slots view.
 */
function uc_deliverytimeslot_view() {
  $sign_flag = variable_get('uc_sign_after_amount', FALSE);
  $currency_sign = variable_get('uc_currency_sign', '$');

  if (isset($_POST['filter_status'])) {
    $_SESSION['filter_status'] = check_plain($_POST['filter_status']);
    $status = check_plain($_POST['filter_status']);
  }
  else {
    if (isset($_SESSION['filter_status'])) {
      $status = $_SESSION['filter_status'];
    }
    else {
      $status = 1;
    }
  }

  if (isset($_POST['start_date'])) {
    $_SESSION['start_date'] = $_POST['start_date'];
    $_SESSION['end_date'] = $_POST['end_date'];
    $start_date = _uc_deliverytimeslot_date($_POST['start_date']);
    $end_date = _uc_deliverytimeslot_date($_POST['end_date']);
  }
  else {
    if (isset($_SESSION['start_date']) && !is_null($_SESSION['start_date'])) {
      $start_date = _uc_deliverytimeslot_date($_SESSION['start_date']);
      $end_date = _uc_deliverytimeslot_date($_SESSION['end_date']);
    }
    else {
      $start_date = _uc_deliverytimeslot_date(FALSE);
      $end_date = _uc_deliverytimeslot_date(FALSE, 15);
    }
  }

  $filter = '<div class="uc-deliverytimeslot-filter-form">'. drupal_get_form('uc_deliverytimeslot_view_filter') .'</div>';
  $filter .= '<div class="uc-deliverytimeslot-filter-date-form">'. drupal_get_form('uc_deliverytimeslot_view_date') .'</div>';

  $timeslot = _uc_deliverytimeslot_timeslot(NULL, TRUE);
  $sql = "SELECT order_id, date, sid FROM {uc_deliverytimeslot_order} ".
         "WHERE date >= '%s' AND date <= '%s' AND confirmed = %d ".
         "ORDER BY date ASC, sid ASC";
  $req = db_query($sql, $start_date, $end_date, (int)$status);
  while ($res = db_fetch_array($req)) {
    $order = uc_order_load($res['order_id']);
    $product_listing = '';
    foreach ($order->products as $product) {
      $product_listing .=  $product->qty .'x '. $product->title .'<br/>';
      if (is_array($product->data['attributes']) && count($product->data['attributes']) > 0) {
        $product_listing .= '<ul class="product-description">';
        foreach ($product->data['attributes'] as $attribute => $option) {
          $product_listing .= '<li>'. t('@attribute: @options', array('@attribute' => $attribute, '@options' => implode(', ', (array)$option))) .'</li>';
        }
        $product_listing .= '</ul>';
      }
    }
    $product_listing .= t("Total order #!orderid:", array('!orderid' => l($order->order_id, 'admin/store/orders/'. $order->order_id))) .' ';
    $sign_flag ? '' : $product_listing .= $currency_sign;
    $product_listing .= uc_order_get_total($order);
    $sign_flag ? $product_listing .= $currency_sign : '';
    $ship_information = uc_order_address($order, 'delivery') .'<br />'. check_plain($order->delivery_phone);
    $date = uc_date_format(date('Y', $res['date']), date('m', $res['date']), date('d', $res['date']));
    $row = array($date, $timeslot[$res['sid']], $ship_information, $product_listing);
    $rows[] = $row;
    $main_rows[] = array_merge($row, array(uc_order_actions($order, TRUE)));
  }
  $header = array(t('Date'), t('Time slot'), t('Delivery address'), t('Order'));
  $main_header = array_merge($header, array(t('Operations')));
  if (arg(4) && arg(4) == 'print') {
    print theme('table', $header, $rows);
    exit();
  }
  $output = $filter . theme('table', $main_header, $main_rows, array('class' => 'uc-deliverytimeslot-table order-pane-table'));
  $icon = '<img src="'. base_path() . drupal_get_path('module', 'uc_deliverytimeslot') .'/images/print_icon.gif" />';
  $output .= l($icon .'&nbsp;'. t('Printable delivery time slots'), 'admin/store/orders/deliverytimeslot/print', array('html' => TRUE));

  return $output;
}

/**
 * @see uc_deliverytimeslot_view()
 */
function uc_deliverytimeslot_view_filter() {
  $options = array(1 => t('Confirmed'), 0 => t('Unconfirmed'));

  if (!isset($_SESSION['filter_status'])) {
    $default_status = 1;
  }
  else {
    $default_status = $_SESSION['filter_status'];
  }

  $form['filter_status'] = array(
    '#type' => 'select',
    '#title' => t('Filter by status'),
    '#description' => t('Confirmed: list all time slots where order status is <em>in checkout</em> or next.') .'<br/>'. t('Unconfirmed: list all time slots where order status is <em>pending</em> or <em>cancel</em>.'),
    '#options' => $options,
    '#default_value' => $default_status,
    '#attributes' => array('onchange' => 'this.form.submit();')
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'View',
    '#attributes' => array('style' => 'display: none;')
  );

  return $form;
}

/**
 * @see uc_deliverytimeslot_view()
 */
function uc_deliverytimeslot_view_date() {
  $form = array();
  if (isset($_SESSION['start_date']) && isset($_SESSION['end_date'])) {
    $start_date = _uc_deliverytimeslot_date($_SESSION['start_date']);
    $end_date = _uc_deliverytimeslot_date($_SESSION['end_date']);
  }
  else {
    $start_date = _uc_deliverytimeslot_date(FALSE);
    $end_date = _uc_deliverytimeslot_date(FALSE, 15);
  }
  $form['date']= array(
    '#type' => 'fieldset',
    '#title' => t('Date range selection'),
    '#collapsible' => TRUE,
    '#collapsed' => isset($_SESSION['start_date']) ? FALSE : TRUE,
  );
  $form['date']['start_date'] = array(
    '#type' => 'date',
    '#title' => t('Start date'),
    '#prefix' => '<div style="display:block;float:left;margin-right:15px;">',
    '#suffix' => '</div>',
    '#default_value' => array(
      'month' => date("n", $start_date),
      'day' => date("j", $start_date),
      'year' => date("Y", $start_date),
    ),
  );
  $form['date']['end_date'] = array(
    '#type' => 'date',
    '#title' => t('End date'),
    '#prefix' => '<div style="display:block;float:left;margin-right:15px;">',
    '#suffix' => '</div>',
    '#default_value' => array(
      'month' => date("n", $end_date),
      'day' => date("j", $end_date),
      'year' => date("Y", $end_date),
    ),
  );
  $form['date']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Apply'),
    '#prefix' => '<div style="display:block;float:left;margin-top:30px;">',
    '#suffix' => '</div>',
  );
  return $form;
}

/**
 * @see uc_deliverytimeslot_view()
 */
function uc_deliverytimeslot_view_date_validate($form, &$form_state) {
  if (isset($form_state['values']['start_date']) && isset($form_state['values']['start_date'])) {
    $start_date = _uc_deliverytimeslot_date($form_state['values']['start_date']);
    $end_date = _uc_deliverytimeslot_date($form_state['values']['end_date']);
    if ($end_date < $start_date) {
      form_set_error('end_date', t('End date needs to be next or equal to start date.'));
    }
  }
}

/**
 * Delivery time slots configuration.
 *
 * @ingroup forms
 * @see uc_deliverytimeslot_admin_view_filter()
 *
 */
function uc_deliverytimeslot_admin_view() {
  drupal_add_css(drupal_get_path('module', 'uc_deliverytimeslot') .'/uc_deliverytimeslot.admin.css');
  $output = '';
  $rows = array();
  $week_days = _uc_deliverytimeslot_week_days();
  $timeslot = $existing_timeslot = _uc_deliverytimeslot_timeslot(NULL, TRUE);
  $shipping_methods = uc_quote_shipping_method_options();

  if ($_POST['filter_day']) {
    $_SESSION['filter_day'] = check_plain($_POST['filter_day']);
    $day = check_plain($_POST['filter_day']);
  }
  else {
    if (isset($_SESSION['filter_day']) && !is_null($_SESSION['filter_day'])) {
      $day = $_SESSION['filter_day'];
    }
    else {
      $day = key(variable_get('uc_deliverytimeslot_week_days', _uc_deliverytimeslot_week_days()));
    }
  }

  if ($_POST['filter_quote']) {
    $_SESSION['filter_quote'] = check_plain($_POST['filter_quote']);
    $method = check_plain($_POST['filter_quote']);
  }
  else {
    if (isset($_SESSION['filter_quote']) && !is_null($_SESSION['filter_quote'])) {
      $method = $_SESSION['filter_quote'];
    }
    else {
      $method = key($shipping_methods);
    }
  }
  // filters
  $output .= '<div class="uc-deliverytimeslot-default">';
  $output .= '<div class="uc-deliverytimeslot-filter-form">'. drupal_get_form('uc_deliverytimeslot_admin_view_filter') .'</div>';

  // default time slots table
  if ($day == 'all') {
    // all days grouped
    $headers = array(t('Day'), t('Time slot'), t('Limit'), t('Operations'));
    $req = db_query("SELECT udd.ddid, udd.day, udd.sid, udd.max_limit
                     FROM {uc_deliverytimeslot_default} udd
                     WHERE udd.quote = '%s'", $method);
    while ($res = db_fetch_object($req)) {
      $row = array();
      $row[] = check_plain($week_days[$res->day]);
      $row[] = $timeslot[$res->sid];
      $row[] = $res->max_limit;
      $row[] = l(t('edit'), 'admin/store/settings/deliverytimeslot/admin/default/'. $res->ddid);
      $rows[] = $row;
    }
    // sort with "human week days order"
    // @todo need to sort by time slot weight for each day
    usort($rows, '_uc_deliverytimeslot_sort');
    $output .= theme('table', $headers, $rows);
  }
  else {
    $headers = array(t('Time slot'), t('Limit'), t('Operations'));
    $req = db_query("SELECT udd.ddid, udd.sid, udd.max_limit, uds.weight
                     FROM {uc_deliverytimeslot_default} udd
                     LEFT JOIN {uc_deliverytimeslot_slots} uds ON uds.sid = udd.sid
                     WHERE udd.day = '%s'
                     AND udd.quote = '%s'
                     ORDER BY uds.weight ASC", $day, $method);

    while ($res = db_fetch_object($req)) {
      $row = array();
      $row[] = $timeslot[$res->sid];
      $row[] = $res->max_limit;
      $row[] = l(t('edit'), 'admin/store/settings/deliverytimeslot/admin/default/'. $res->ddid);
      $rows[] = $row;
    }
    if (count($rows)) {
      $output .= theme('table', $headers, $rows);
    }
    else {
      $output .= '<p><em>'. t('No default time slots table for this shipping method on this day.') .'</em></p>';
    }
  }

  // add time slot
  $existing_slot_query = db_query("SELECT sid FROM {uc_deliverytimeslot_default} WHERE day = '%s' AND quote = '%s'", $day, $$method);
  while ($existing_slot_res = db_fetch_array($existing_slot_query)) {
    unset($existing_timeslot[$existing_slot_res['sid']]);
  }
  if (count($existing_timeslot) > 0) {
    if ($day == 'all') {
      $output .= '<p>'. t('You need to select a day to add a time slot') .'</p>';
    }
    else {
      $output .= '<p>'. l(t('Add a time slot for !quote on !day', array('!quote' => $shipping_methods[$method], '!day' => $week_days[$day])), 'admin/store/settings/deliverytimeslot/admin/default/add/'. $method .'/'. $day) .'</p>';
    }
  }
  elseif (count($rows)) {
    $output .= '<p>'. t('All time slots for !day used.', array('!day' => $day)) .'</p>';
  }
  $output .= '</div>';

  // special table time
  $req = db_query("SELECT * FROM {uc_deliverytimeslot_special}
                   WHERE uc_deliverytimeslot_special.quote = '%s'
                   ORDER BY uc_deliverytimeslot_special.date ASC", $method);
  $rows = array();
  while ($res = db_fetch_object($req)) {
    $row = array();
    $row[] = uc_date_format(date('Y', $res->date), date('m', $res->date), date('d', $res->date));
    $row[] = $timeslot[$res->sid];
    $row[] = $res->max_limit;
    $row[] = l(t('edit'), 'admin/store/settings/deliverytimeslot/admin/special/'. $res->dsid);
    $rows[] = $row;
  }
  $output .= '<div class="uc-deliverytimeslot-special">';
  if (count($rows)) {
    $headers = array(t('Date'), t('Time slot'), t('Limit'), t('Operations'));
    $output .= theme('table', $headers, $rows, array(), t('Special delivery time slots table'));
  }
  else {
    $output .= '<p><em>'. t('No special date found.') .'</em></p>';
  }

  // add a special date
  $output .= '<p>'. l(t('Add a special date'), 'admin/store/settings/deliverytimeslot/admin/special/add') .'</p>';
  $output .= '</div>';

  return $output;
}

/**
 * @see uc_deliverytimeslot_admin_view()
 */
function uc_deliverytimeslot_admin_view_filter() {
  $form = array();
  $week_days = variable_get('uc_deliverytimeslot_week_days', _uc_deliverytimeslot_week_days());
  if (isset($_SESSION['filter_day'])) {
    $default_day = $_SESSION['filter_day'];
  }
  else {
    $default_day = 'monday';
  }
  if (isset($_SESSION['filter_quote'])) {
    $filter_quote = $_SESSION['filter_quote'];
  }
  $form['filter_quote'] = array(
    '#type' => 'select',
    '#title' => t('Shipping method'),
    '#options' => uc_quote_shipping_method_options(),
    '#default_value' => $filter_quote,
    '#attributes' => array('onchange' => 'this.form.submit();'),
  );
  $form['filter_day'] = array(
    '#type' => 'select',
    '#title' => t('Day'),
    '#options' => $week_days + array('all' => t('All')),
    '#default_value' => $default_day,
    '#attributes' => array('onchange' => 'this.form.submit();'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'View',
    '#attributes' => array('style' => 'display:none;')
  );
  return $form;
}

/**
 * Edit/Add form for delivery time slots and special date
 *
 * @ingroup forms
 * @see uc_deliverytimeslot_admin_edit_form_validate()
 * @see uc_deliverytimeslot_admin_edit_form_submit()
 *
 */
function uc_deliverytimeslot_admin_edit_form($form_state, $type = 'default', $id = 0) {
  $form = array();
  $time_slot = _uc_deliverytimeslot_timeslot(NULL, TRUE);
  $shipping_methods = uc_quote_shipping_method_options();
  $week_days = _uc_deliverytimeslot_week_days();

  // edit mode
  if (is_numeric($id)) {
    switch ($type) {
      case 'default':
        $req = db_fetch_array(db_query("SELECT * FROM {uc_deliverytimeslot_default} WHERE ddid = %d", $id));
        $form['ddid'] = array('#type' => 'value', '#value' => $req['ddid']);
        $day = $req['day'];
        break;
      case 'special':
        $req = db_fetch_array(db_query("SELECT * FROM {uc_deliverytimeslot_special} WHERE dsid = %d", $id));
        $form['dsid'] = array('#type' => 'value', '#value' => $req['dsid']);
        $form['date'] = array('#type' => 'value', '#value' => $req['date']);
        $form['sid'] = array(
          '#type' => 'select',
          '#title' => t('Time slots'),
          '#description' => t('Hold Ctrl + click to select multiple values.'),
          '#default_value' => $req['sid'],
          '#options' => $time_slot,
          '#size' => count($time_slot),
        );
        $day = date(variable_get('uc_date_format_default', 'm/d/Y'), $req['date']);
        break;
    }
    drupal_set_title(t('Edit !type !method for !day | !timeslot', array('!type' => $type, '!method' => $shipping_methods[$req['quote']], '!day' => $week_days[$day], '!timeslot' => $time_slot[$req['sid']])));
    $max = $req ['max_limit'];
    $mode = t('Submit');
  }
  // add mode
  else {
    switch ($type) {
      case 'default':
        $sid = check_plain(arg(7));
        $day = check_plain(arg(8));
        drupal_set_title(t('Add a time slot for !quote on !day', array('!quote' => $shipping_methods[$sid], '!day' => $week_days[$day])));
        $form['day'] = array('#type' => 'value', '#value' => $day);
        $existing_slot_query = db_query("SELECT sid FROM {uc_deliverytimeslot_default} WHERE quote = '%s' AND day = '%s'", $sid, $day);
        while ($existing_slot_res = db_fetch_array($existing_slot_query)) {
          unset($time_slot[$existing_slot_res['sid']]);
        }
        if (count($time_slot) > 0) {
          $form['sid'] = array(
            '#type' => 'select',
            '#multiple' => (count($time_slot) > 1) ? TRUE : FALSE,
            '#description' => t('Hold Ctrl + click to select multiple values.'),
            '#title' => t('Time slots'),
            '#options' => $time_slot,
            '#size' => count($time_slot),
            '#required' => TRUE,
          );
        }
        $form['quote'] = array(
          '#type' => 'hidden',
          '#value' => $sid,
        );
        break;
      case 'special':
        drupal_set_title(t('Add a special date'));
        $form['quote'] = array(
          '#type' => 'select',
          '#title' => t('Shipping quote'),
          '#default_value' => isset($_SESSION['filter_quote']) ? $_SESSION['filter_quote'] : '',
          '#description' => t('Which method is concerned by this special date.'),
          '#options' => uc_quote_shipping_method_options(),
          '#required' => TRUE,
        );
        $form['date'] = array(
          '#type' => 'date',
          '#title' => t('Date'),
        );
        $form['sid'] = array(
          '#type' => 'select',
          '#multiple' => TRUE,
          '#required' => TRUE,
          '#title' => t('Time slots'),
          '#description' => t('Hold Ctrl + click to select multiple values.'),
          '#options' => $time_slot,
          '#size' => count($time_slot),
        );
        break;
    }
    $mode = t('Add');
  }
  $form['max_limit'] = array(
    '#type' => 'textfield',
    '#title' => t('Limit'),
    '#description' => t('How many customers can choose this time slot. Then this time slot will be marked full.'),
    '#default_value' => $max ? $max : 5,
    '#size' => 4,
    '#required' => TRUE,
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => $mode,
  );
  if ($id != 'add') {
    $form['buttons']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
    );
  }
  $form['buttons']['cancel'] = array(
    '#value' => l(t('Cancel'), 'admin/store/settings/deliverytimeslot'),
  );
  return $form;
}

/**
 * @see uc_deliverytimeslot_admin_edit_form()
 */
function uc_deliverytimeslot_admin_edit_form_validate($form, &$form_state) {
  if (!empty($form_state['values']['max_limit']) && !is_numeric($form_state['values']['max_limit'])) {
    form_set_error('max_limit', t('The limit must be numeric.'));
  }
  // no duplicate on special slots
  if ($form_state['values']['op'] == t('Submit')) {
    if (!is_array($form_state['values']['sid']) && isset($form_state['values']['sid'])) {
      $sql = "SELECT count(*) FROM {uc_deliverytimeslot_special} WHERE date= %d AND sid = %d AND dsid <> %d";
      if (db_result(db_query($sql, $form_state['values']['date'], (int)$form_state['values']['sid'], $form_state['values']['dsid']))) {
        form_set_error('sid', t('This time slot on this day already exist.'));
      }
    }
  }
}

/**
 * @see uc_deliverytimeslot_admin_edit_form()
 */
function uc_deliverytimeslot_admin_edit_form_submit($form, &$form_state) {
  switch ($form_state['values']['op']) {
    case t('Delete'):
      // Delete time slot
      if ($form_state['values']['ddid']) {
        db_query("DELETE FROM {uc_deliverytimeslot_default} WHERE ddid = %d", $form_state['values']['ddid']);
      }
      // Delete special
      if ($form_state['values']['dsid']) {
        db_query("DELETE FROM {uc_deliverytimeslot_special} WHERE dsid = %d", $form_state['values']['dsid']);
      }
      $msg = t('deleted');
      $form_state['redirect'] = 'admin/store/settings/deliverytimeslot';
      break;
    case t('Submit'):
      // Update time slot
      if ($form_state['values']['ddid']) {
        db_query("UPDATE {uc_deliverytimeslot_default} SET max_limit = %d WHERE ddid = %d", $form_state['values']['max_limit'], $form_state['values']['ddid']);
      }
      // Update special
      if ($form_state['values']['dsid']) {
        db_query("UPDATE {uc_deliverytimeslot_special} SET max_limit = %d, sid = %d WHERE dsid = %d", $form_state['values']['max_limit'], $form_state['values']['sid'], $form_state['values']['dsid']);
      }
      $msg = t('updated');
      $form_state['redirect'] = 'admin/store/settings/deliverytimeslot';
      break;
    case t('Add'):
      $msg = t('created');
      if (isset($form_state['values']['sid'])) {
        $form_sid = $form_state['values']['sid'];
        $max = $form_state['values']['max_limit'];
        $quote = $form_state['values']['quote'];
        if (isset($form_state['values']['day'])) {
          $sql = "INSERT INTO {uc_deliverytimeslot_default} (quote, day, sid, max_limit) VALUES ('%s', '%s', %d, %d)";
          $day = $form_state['values']['day'];
        }
        else {
          $day = _uc_deliverytimeslot_date($form_state['values']['date']);
          $sql = "INSERT INTO {uc_deliverytimeslot_special} (quote, date, sid, max_limit) VALUES ('%s', %d, %d, %d)";
        }
        if (is_array($form_sid)) {
          foreach ($form_sid AS $sid) {
            db_query($sql, $quote, $day, $sid, $max);
          }
        }
        else {
          db_query($sql, $quote, $day, $form_sid, $max);
        }
      }
      $form_state['redirect'] = 'admin/store/settings/deliverytimeslot';
      break;
  }
  drupal_set_message(t('Delivery time slot !msg.', array('!msg' => $msg)));
}

/**
 * Delivery time slots settings on checkout admin page
 */
function uc_deliverytimeslot_settings_pane() {
  $form = array();
  $form['uc_deliverytimeslot_display_text'] = array(
    '#type' => 'textarea',
    '#title' => t('Description in delivrey checkout pane'),
    '#default_value' => variable_get('uc_deliverytimeslot_display_text', t('Please specify a date and time of delivery.')),
    '#description' => t('Use this text to give special information about delivrey time to your customer.'),
  );
  $form['uc_deliverytimeslot_display_legend'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show legend'),
    '#default_value' => variable_get('uc_deliverytimeslot_display_legend', 1),
    '#description' => t('Display legend table under time slots table.'),
  );
  $form['uc_deliverytimeslot_display_show'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show pane at start'),
    '#default_value' => variable_get('uc_deliverytimeslot_display_show', 0),
    '#description' => t('Pane is hidden by default and showing up only when shipping quote is selected, but you can force pane visibility with this checkbox.'),
  );
  $form['uc_deliverytimeslot_display_days'] = array(
    '#type' => 'textfield',
    '#title' => t('Days on pane'),
    '#default_value' => variable_get('uc_deliverytimeslot_display_days', 5),
    '#description' => t('How days in future your customer can see and select.'),
    '#size' => 2,
    '#maxlength' => 2,
    '#required' => TRUE,
    '#element_validate' => array('_uc_deliverytimeslot_validate_numeric'),
  );
  $form['uc_deliverytimeslot_display_days_future'] = array(
    '#type' => 'textfield',
    '#title' => t('Days offset'),
    '#default_value' => variable_get('uc_deliverytimeslot_display_days_future', 0),
    '#description' => t('Select first day your customer can choose, 0 = today, 1 = tomorrow, etc.'),
    '#size' => 2,
    '#maxlength' => 2,
    '#required' => TRUE,
    '#element_validate' => array('_uc_deliverytimeslot_validate_numeric'),
  );
  $form['uc_deliverytimeslot_display_hours_future'] = array(
    '#type' => 'textfield',
    '#title' => t('Hours offset'),
    '#default_value' => variable_get('uc_deliverytimeslot_display_hours_future', 2),
    '#description' => t('Only used if Days offset is 0.<br/>0 = if site time is 10:00, all time slots that begin 10:00 will not be available.<br/>1 or more = add offset, is site time is 10:00 with an offset of 1, all time slots that begin before 11:00 will not be available.'),
    '#size' => 2,
    '#maxlength' => 2,
    '#required' => TRUE,
    '#element_validate' => array('_uc_deliverytimeslot_validate_numeric'),
  );
  $form['uc_deliverytimeslot_display_mode'] = array(
    '#type' => 'select',
    '#title' => t('Configure hedear/row of delivery time slots table'),
    '#default_value' => variable_get('uc_deliverytimeslot_display_mode', 'day'),
    '#description' => t('Choose display of the table, days or time slots on header.'),
    '#options' => array(
                    'day' => t('Day on header row / time slots on left column'),
                    'timeslot' => t('Time slots on header row / days on left column')
                    ),
  );
  return $form;
}

/**
 * Delivery time slots settings form
 *
 * @ingroup forms
 * @see uc_deliverytimeslot_settings_form_validate()
 * @see uc_deliverytimeslot_settings_form_submit()
 */
function uc_deliverytimeslot_settings_form(&$form_state) {
  $form = array();
  $timeslots = _uc_deliverytimeslot_timeslot();
  $options_hours = _uc_deliverytimeslot_options_time();
  $options_minute = _uc_deliverytimeslot_options_time(59);

  $form['timeslots'] = array(
    '#type' => 'fieldset',
    '#collapsed' => FALSE,
    '#collapsible' => FALSE,
    '#title' => t('Time slots'),
  );
  $form['timeslots']['rows']['#tree'] = TRUE;
  $form['timeslots']['rows']['#theme'] = 'uc_deliverytimeslot_timeslots';
  foreach ($timeslots AS $sid => $slot) {
    $form['timeslots']['rows'][$sid]['title'] = array(
      '#type' => 'textfield',
      '#default_value' => $slot['title'],
      '#size' => 40,
      '#required' => TRUE,
    );
    $form['timeslots']['rows'][$sid]['start_h'] = array(
      '#type' => 'select',
      '#options' => $options_hours,
      '#default_value' => $slot['start_h'],
    );
    $form['timeslots']['rows'][$sid]['start_m'] = array(
      '#type' => 'select',
      '#options' => $options_minute,
      '#default_value' => $slot['start_m'],
    );
    $form['timeslots']['rows'][$sid]['end_h'] = array(
      '#type' => 'select',
      '#options' => $options_hours,
      '#default_value' => $slot['end_h'],
    );
    $form['timeslots']['rows'][$sid]['end_m'] = array(
      '#type' => 'select',
      '#options' => $options_minute,
      '#default_value' => $slot['end_m'],
    );
    $form['timeslots']['rows'][$sid]['weight'] = array(
      '#type' => 'weight',
      '#default_value' => $slot['weight'],
      '#delta' => 10,
    );
  }
  $form['timeslots']['help'] = array(
    '#value' => '<div class="description">'. t('You can edit and reorder time slots. Start and end time format is 24 hours but display for your customers is handle by free title field.') .'</div>',
    '#suffix' => l(t('Add a time slot'), 'admin/store/settings/deliverytimeslot/timeslot/add/0'),
  );
  $form['days'] = array(
    '#type' => 'fieldset',
    '#collapsed' => FALSE,
    '#collapsible' => FALSE,
    '#title' => t('Settings'),
  );
  $default_value = variable_get('uc_deliverytimeslot_week_days', _uc_deliverytimeslot_week_days());
  $form['days']['days'] = array(
    '#type' => 'checkboxes',
    '#multiple' => TRUE,
    '#title' => t('Week days'),
    '#default_value' => array_keys($default_value),
    '#description' => t('Choose which days will be available for delivery checkout selection.') .'<br/>'. t('<strong>Warning</strong>: all created delivery time slots with unselected days will be deleted.'),
    '#options' => _uc_deliverytimeslot_week_days(),
    '#required' => TRUE,
  );
  $form['days']['display_format'] = array(
    '#type' => 'textfield',
    '#title' => t('Day format'),
    '#description' =>  t('Choose day format used to display.') .'<br/>'. t('Supply a format string using !link syntax.', array('!link' => l(t('PHP date'), 'http://www.php.net/date'))),
    '#default_value' => variable_get('uc_deliverytimeslot_display_format', 'l F j'),
    '#size' => 10,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#prefix' => '<div class="description">'. t('Checkout pane settings is located on <a href="!url">checkout settings</a> page.', array('!url' => url('admin/store/settings/checkout/edit/panes'))) .'</div>',
  );
  return $form;
}

/**
 * @see uc_deliverytimeslot_settings_form()
 */
function uc_deliverytimeslot_settings_form_validate($form, &$form_state) {
  foreach ($form_state['values']['rows'] AS $sid => $values) {
    if ($values['end_h'] < $values['start_h']) {
      form_set_error('rows]['. $sid .'][end_h', t('End time must be after start!'));
    }
    if (($values['end_h'] == $values['start_h']) && ($values['end_m'] <= $values['start_m'])) {
      form_set_error('rows]['. $sid .'][end_m', t('End time must be after start!'));
    }
  }
}

/**
 * @see uc_deliverytimeslot_settings_form()
 */
function uc_deliverytimeslot_settings_form_submit($form, &$form_state) {
  // time slots
  foreach ($form_state['values']['rows'] AS $sid => $values) {
    $start = $values['start_h'] . sprintf('%02d', $values['start_m']);
    $end = $values['end_h'] . sprintf('%02d', $values['end_m']);
    db_query("UPDATE {uc_deliverytimeslot_slots}
              SET title = '%s', start = %d, end = %d, weight = %d
              WHERE sid = %d", $values['title'], $start, $end, $values['weight'], $sid);
  }
  // days
  $week_days = _uc_deliverytimeslot_week_days();
  $variable_week_days = array();
  foreach ($form_state['values']['days'] AS $key => $value) {
    if ($value) {
      $variable_week_days[$value] = $week_days[$value];
    }
    else {
      // delete existing time slots table by shipping method for this day
      db_query("DELETE FROM {uc_deliverytimeslot_default} WHERE day = '%s'", $key);
    }
  }
  variable_set('uc_deliverytimeslot_week_days', $variable_week_days);
  variable_set('uc_deliverytimeslot_display_format', $form_state['values']['display_format']);
  drupal_set_message(t("Settings saved."));
}

/*
 * Theme the time slots settings form
 *
 * @ingroup themeable
 * @see uc_deliverytimeslot_settings_form()
 */
function theme_uc_deliverytimeslot_timeslots($form) {
  drupal_add_tabledrag('deliverytimeslot-slots', 'order', 'sibling', 'weight');
  $header = array(t('Title'), array('data' => t('Start time'), 'colspan' => 4), array('data' => t('End time'), 'colspan' => 3), t('Action'), t('Weight'));
  $rows = array();
  foreach (element_children($form) as $key) {
    // Add class to group weight fields for drag and drop.
    $form[$key]['weight']['#attributes']['class'] = 'weight';
    $row = array();
    $row[] = drupal_render($form[$key]['title']);
    $row[] = drupal_render($form[$key]['start_h']);
    $row[] = ':';
    $row[] = drupal_render($form[$key]['start_m']);
    $row[] = t('to');
    $row[] = drupal_render($form[$key]['end_h']);
    $row[] = ':';
    $row[] = drupal_render($form[$key]['end_m']);
    $row[] = l(t('delete'), 'admin/store/settings/deliverytimeslot/timeslot/delete/'. $key);
    $row[] = drupal_render($form[$key]['weight']);
    $rows[$key] = array('data' => $row, 'class' => 'draggable');
  }
  $output = theme('table', $header, $rows, array('id' => 'deliverytimeslot-slots'));
  $output .= drupal_render($form);
  return $output;
}

/**
 * Generate a default time slots table.
 *
 * @ingroup forms
 * @see uc_deliverytimeslot_settings_generate_validate()
 * @see uc_deliverytimeslot_settings_generate_submit()
 *
 */
function uc_deliverytimeslot_settings_generate() {
  $form = array();
  $form['info'] = array(
    '#type' => 'item',
    '#value' => t('<b>Notice</b>: generate a new table for a shipping method will erase existing one!'),
  );
  $form['quotes'] = array(
    '#type' => 'select',
    '#multiple' => TRUE,
    '#title' => t('Shipping method'),
    '#description' => t('On which shipping method this time slots will be generated.') .'<br/>'. t('Hold Ctrl + click to select multiple values.'),
    '#options' => uc_quote_shipping_method_options(),
    '#required' => TRUE,
  );
  $days = variable_get('uc_deliverytimeslot_week_days', _uc_deliverytimeslot_week_days());
  $form['days'] = array(
    '#type' => 'select',
    '#multiple' => TRUE,
    '#title' => t('Week days'),
    '#description' => t('Choose wich days will be used for generation.') .'<br/>'. t('Hold Ctrl + click to select multiple values.'),
    '#options' => $days,
    '#required' => TRUE,
    '#size' => count($days),
  );
  $options = _uc_deliverytimeslot_timeslot(NULL, TRUE);
  $form['timeslot'] = array(
    '#type' => 'select',
    '#multiple' => TRUE,
    '#title' => t('Time slots'),
    '#description' => t('Choose wich time slot will be used for generation.') .'<br/>'. t('Hold Ctrl + click to select multiple values.'),
    '#options' => $options,
    '#required' => TRUE,
    '#size' => count($options) < 15 ? count($options) : 15,
  );
  $form['max_limit'] = array(
    '#type' => 'textfield',
    '#title' => t('Order limit'),
    '#description' => t('How many customers can choose this time slot. Then this time slot will be marked full.'),
    '#default_value' => 5,
    '#size' => 4,
    '#required' => TRUE,
    '#element_validate' => array('_uc_deliverytimeslot_validate_numeric'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Generate time slots table'),
  );
  return $form;
}

/**
 * @see uc_deliverytimeslot_settings_generate()
 */
function uc_deliverytimeslot_settings_generate_submit($form, &$form_state) {
  if ($form_state['values']['op'] == t('Generate time slots table')) {
    $days = $form_state['values']['days'];
    $timeslot = $form_state['values']['timeslot'];
    $max_limit = $form_state['values']['max_limit'];
    $shipping_quotes = $form_state['values']['quotes'];

    foreach ($shipping_quotes as $shipping_quote) {
      db_query("DELETE FROM {uc_deliverytimeslot_default} WHERE quote = '%s'", $shipping_quote);
      foreach ($days as $day) {
        foreach ($timeslot as $sid) {
          db_query("INSERT INTO {uc_deliverytimeslot_default} (quote, day, sid, max_limit) VALUES ('%s', '%s', %d, %d)", $shipping_quote, $day, $sid, $max_limit);
        }
      }
    }
    drupal_set_message(t('Delivery time slots table generated.'));
  }
}

/**
 * Add a time slot.
 *
 * @ingroup forms
 * @see uc_deliverytimeslot_add_timeslot_form_validate()
 * @see uc_deliverytimeslot_add_timeslot_form_submit()
 *
 */
function uc_deliverytimeslot_add_timeslot_form($form_state, $mode = 'add', $id = NULL) {
  drupal_add_css(drupal_get_path('module', 'uc_deliverytimeslot') .'/uc_deliverytimeslot.admin.css');
  $form = array();
  if ($mode == 'delete' && $id) {
    drupal_set_title(t('Delete a time slot'));
    $form['sid'] = array(
      '#type' => 'hidden',
      '#value' => $id,
    );
    return confirm_form($form, t('Do you want to delete this time slot?'),
    'admin/store/settings/deliverytimeslot/settings',
    t('This will remove the time slot and any delivery time slots table or special date with this time slot. This action can not be undone.'),
    t('Delete'));
  }

  $options_hours = _uc_deliverytimeslot_options_time();
  $options_minute = _uc_deliverytimeslot_options_time(59);
  $form['title'] = array(
    '#title' => t('Title'),
    '#description' => t('Timeslot title show to your customers.'),
    '#type' => 'textfield',
    '#size' => 20,
    '#required' => TRUE,
  );
  $form['start_h'] = array(
    '#title' => t('Start time'),
    '#type' => 'select',
    '#options' => $options_hours,
    '#prefix' => '<div class="time-select">',
    '#suffix' => ':',
  );
  $form['start_m'] = array(
    '#type' => 'select',
    '#options' => $options_minute,
    '#suffix' => '</div>',
  );
  $form['end_h'] = array(
    '#title' => t('End time'),
    '#type' => 'select',
    '#options' => $options_hours,
    '#prefix' => '<div class="time-select">',
    '#suffix' => ':',
  );
  $form['end_m'] = array(
    '#type' => 'select',
    '#options' => $options_minute,
    '#suffix' => '</div>',
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#suffix' => l(t('Go back'), 'admin/store/settings/deliverytimeslot/settings'),
  );
  return $form;
}

/**
 * @see uc_deliverytimeslot_add_timeslot_form()
 */
function uc_deliverytimeslot_add_timeslot_form_validate($form, &$form_state) {
  if ($form_state['values']['op'] != t('Delete')) {
    if ($form_state['values']['end_h'] < $form_state['values']['start_h']) {
      form_set_error('end_h', t('End time must be after start!'));
    }
    if (($form_state['values']['end_h'] == $form_state['values']['start_h']) && ($form_state['values']['end_m'] <= $form_state['values']['start_m'])) {
      form_set_error('end_m', t('End time must be after start!'));
    }
  }
}

/**
 * @see uc_deliverytimeslot_add_timeslot_form()
 */
function uc_deliverytimeslot_add_timeslot_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] == t('Delete')) {
    db_query("DELETE FROM {uc_deliverytimeslot_slots} WHERE sid = %d", $form_state['values']['sid']);
    db_query("DELETE FROM {uc_deliverytimeslot_special} WHERE sid = %d", $form_state['values']['sid']);
    db_query("DELETE FROM {uc_deliverytimeslot_default} WHERE sid = %d", $form_state['values']['sid']);
    drupal_set_message(t("Time slot deleted."));
    drupal_goto('admin/store/settings/deliverytimeslot/settings');
  }
  else {
    $start = $form_state['values']['start_h'] . sprintf('%02d', $form_state['values']['start_m']);
    $end = $form_state['values']['end_h'] . sprintf('%02d', $form_state['values']['end_m']);
    db_query("INSERT INTO {uc_deliverytimeslot_slots} (start, end, title) VALUES (%d, %d, '%s')", $start, $end, $form_state['values']['title']);
    drupal_set_message(t("Time slot saved."));
  }
}